package refactor;

import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;

import mixin.AST_FieldDecl;
import mixin.AST_FieldDeclElem;
import mixin.AST_Modifiers;
import mixin.AST_ModifiersElem;
import mixin.AST_ParList;
import mixin.AstCursor;
import mixin.AstNode;
import mixin.AstOptNode;
import mixin.AstToken;
import mixin.ClassBodyDeclaration;
import mixin.ClsBody;
import mixin.DecNameDim;
import mixin.FormParDecl;
import mixin.MDSBlock;
import mixin.MethodDcl;
import mixin.ModAbstract;
import mixin.MthDector;
import mixin.QName;
import mixin.TypeDeclaration;
import mixin.program;




/**
 * Implementiert das Interface MethodInfo und bezieht die Informationen aus dem
 * AST.
 * 
 * @layer<typesys>
 * @author Stephan Klapproth
 * 
 */
abstract class MethodInfoAst$$typesys extends AbstractMethodInfo implements AstElement {

    private ClassInfo clazz;

    private ClassInfo returnType;

    private FieldInfo[] params;

    ModifierInfo[] modifier;

    ClassBodyDeclaration astClassBodyDcl;

    private BlockCScope blockScope;
    
    /**
     * Konstruktor.
     * 
     * @param aDecl
     *            MethodDcl AST Knoten.
     */
    public MethodInfoAst$$typesys(ClassBodyDeclaration aDecl) {
        astClassBodyDcl = aDecl;
        modifier = TypesysUtil.getModifierFromAst(getModifiersASTNode());
        initName();
        initStatements();
    }
    
    void isValidClassBodyDcl() {
        if (!(astClassBodyDcl instanceof MethodDcl)) {
            throw new IllegalArgumentException("aDecl has to be of type MethodDcl");
        }
    }
    
    AST_Modifiers getModifiersASTNode() {
        return ((MethodDcl) astClassBodyDcl).getAST_Modifiers();
    }
	
    public ModifierInfo[] getModifier() {
        return modifier;
    }
    
    void initStatements() {
        MethodDcl tmpDcl = (MethodDcl) astClassBodyDcl;
        if (!(tmpDcl.getMethodDeclSuffix() instanceof MDSBlock)) {
            return;
        }
        MDSBlock tmpMDSBlock = (MDSBlock) tmpDcl.getMethodDeclSuffix();
        blockScope = (BlockCScope) AbstractBlockScope.newInstance(tmpMDSBlock
                .getBlock());
        blockScope.setContext(((MethodInfoAst) this));
    }
    
    private boolean isAbstract() {
        if (clazz.isInterface()) {
            return true;
        }
        return Arrays.asList(getModifier()).contains(
                ModifierInfo.valueOf(ModifierInfo.ABSTRACT));
    }

    public AstNode getAstNode() {
        return astClassBodyDcl;
    }

    /*
     * (non-Javadoc)
     * 
     * @see AstElement#startLineNum()
     */
    public int startLineNum() {
        AstToken tmpTok = getAstNode().findToken();
        if (tmpTok == null) {
            throw new TypesysException("Can't get the first token of "
                    + getAstNode().className());
        }
        return tmpTok.lineNum();
    }

    /*
     * (non-Javadoc)
     * 
     * @see AstElement#endLineNum()
     */
    public int endLineNum() {
        AstToken tmpTok = TypesysUtil.findLastToken(getAstNode());
        if (tmpTok == null) {
            throw new TypesysException("Can't get the last token of "
                    + getAstNode().className());
        }
        return tmpTok.lineNum();
    }
    
    BlockScope[] getBlockScopes() {
        if (!isAbstract() && blockScope != null) {
            return new BlockScope[] { blockScope };
        }
        return new BlockScope[0];
    }
    
    /**
     * Gibt zurck, ob es sich um eine Methodenverfeinerung handelt.
     * 
     * @return true, wenn es sich um eine Methodenverfeinerung handelt.
     */
    public boolean isRefinement() {
        // wenn Methode in einer Basisklasse, kann sie keine Verfeinerung sein
        if (!(clazz instanceof RefinementInfoAst)) {
            return false;
        }
        ClassInfo tmpClass = clazz;
        while (tmpClass instanceof RefinementInfoAst) {
            tmpClass = tmpClass.getSuperClass();
            // in Klasse nach Methode suchen:
            if (((ClassInfoAst) tmpClass).getMethod(getName(), getParameters()) != null) {
                return true;
            }
        }
        return false;
    }

    /**
     * Gibt alle Methodenaufrufe zurck, die sich innerhalb der Methode,
     * reprsentiert durch dieses Objekt, befinden.
     * 
     * <p>
     * Falls sich in der Methode kein Methodenaufruf befindet, wird ein Array
     * der Lnge 0 zurckgegeben.
     * 
     * @return Array mit allen Methodenaufrufen.
     */
    public MethodInvocation[] getMethodInvocations() {
        if (isAbstract() || blockScope == null) {
            return new MethodInvocation[0];
        }
        return blockScope.getMethodInvocations();
    }
    
    /**
     * Gibt alle Referenzen auf Klassen zurck, die sich innerhalb der Methode,
     * reprsentiert durch dieses Objekt, befinden.
     * 
     * <p>
     * Falls sich in der Methode keine Klassenreferenzen befinden, wird ein Array
     * der Lnge 0 zurckgegeben.
     * 
     * @return Array mit allen Klassenreferenzen.
     */
    public ClassReference[] getClassReferences() {
        if (isAbstract() || blockScope == null) {
            return new ClassReference[0];
        }
        return blockScope.getClassReferences();
    }

    /**
     * Gibt alle Objektattributzugriffe zurck, die sich innerhalb der Methode,
     * reprsentiert durch dieses Objekt, befinden.
     * 
     * <p>
     * Falls sich in der Methode kein Attributzugriff befindet, wird ein Array
     * der Lnge 0 zurckgegeben.
     * 
     * @return Array mit allen Attributzugriffen.
     */
    public FieldAccess[] getFieldAccesses() {
        if (isAbstract() || blockScope == null) {
            return new FieldAccess[0];
        }
        return blockScope.getFieldAccesses();
    }

    private ClassInfo initReturnType() {
        if (getContext() == null) {
            throw new IllegalStateException("Decl. scope for " + getFullName() + " not set!");
        }

        int tmpDim = ArrayType.getDimCount(((MethodDcl) astClassBodyDcl).getAST_TypeName());
        return TypeFactory.valueOf(((MethodDcl) astClassBodyDcl).getAST_TypeName().GetName(),
                getContext(), tmpDim);
    }
    
    AST_ParList getAST_ParListNode() {
        return ((MthDector) ((MethodDcl) astClassBodyDcl)
                .getMethodDeclarator()).getAST_ParList();
    }
    
    private void initParams() {
        AST_ParList tmpParList = getAST_ParListNode();
        if (tmpParList == null) {
            params = new FieldInfo[0];
            return;
        }

        List tmpParams = new ArrayList();
        AstCursor c = new AstCursor();
        for (c.FirstElement(tmpParList); c.MoreElement(); c.NextElement()) {
            FormParDecl tmpParDecl = (FormParDecl) c.node;
            FieldInfo tmpParam = new FieldInfoAst(tmpParDecl.getAST_TypeName(),
                    (DecNameDim) tmpParDecl.getVariableDeclaratorId());
            tmpParams.add(tmpParam);
            tmpParam.setContext(((MethodInfoAst) this));
        }
        params = (FieldInfo[]) tmpParams
                .toArray(new FieldInfo[tmpParams.size()]);
    }
    
    QName getQNameNode() {
        return ((MthDector) ((MethodDcl) astClassBodyDcl).getMethodDeclarator()).getQName();
    }

    private void initName() {
        name = getQNameNode().GetName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see Declaration#getContext()
     */
    public Scope getContext() {
        return clazz;
    }

    /*
     * (non-Javadoc)
     * 
     * @see Declaration#setContext(Scope)
     */
    public void setContext(Scope aScope) {
        if (!(aScope instanceof ClassInfo)) {
            throw new IllegalArgumentException(aScope.getClass().getName()
                    + " is not a valid Scope for a method!");
        }
        clazz = (ClassInfo) aScope;
    }

    /*
     * (non-Javadoc)
     * 
     * @see AbstractNamed#getFullName()
     */
    public String getFullName() {
        return clazz.getFullName() + TypesysUtil.DOT + getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see Scope#addDeclaration(Declaration)
     */
    public void addDeclaration(Declaration aDeclaration) {
        throw new UnsupportedOperationException("Can't add declarations "
                + "to a method.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see MethodInfo#getReturnType()
     */
    public ClassInfo getReturnType() {
        if (returnType == null) {
            returnType = initReturnType();
        }
        return returnType;
    }

    /*
     * (non-Javadoc)
     * 
     * @see MethodInfo#getParameters()
     */
    public ClassInfo[] getParameters() {
        List tmpTypes = new ArrayList();
        for (int i = 0; i < getParams().length; i++) {
            tmpTypes.add(getParams()[i].getType());
        }
        return (ClassInfo[]) tmpTypes.toArray(new ClassInfo[tmpTypes.size()]);
    }

    FieldInfo[] getParams() {
        if (params == null) {
            initParams();
        }
        return params;
    }

    public String signature() {
        return getSignature(getName(), getParameters());
    }

    public FieldInfo findField(String aName) {
        for (int i = 0; i < getParams().length; i++) {
            if (getParams()[i].getName().equals(aName)) {
                return getParams()[i];
            }
        }
        if (clazz == null) {
            throw new IllegalStateException("Declaring class of " + getFullName() + " not set.");
        }
        return clazz.findField(aName);
    }

    public boolean isPublic() {
        return Arrays.asList(getModifier()).contains(
                ModifierInfo.valueOf(ModifierInfo.PUBLIC));
    }

    public boolean isPrivate() {
        return Arrays.asList(getModifier()).contains(
                ModifierInfo.valueOf(ModifierInfo.PRIVATE));
    }

    public boolean isProtected() {
        return Arrays.asList(getModifier()).contains(
                ModifierInfo.valueOf(ModifierInfo.PROTECTED));
    }
}




/**
 * 
 * @layer<refactoring_changemodifier>
 * @author Stephan Kauschka
 * 
 */
 abstract class MethodInfoAst$$refactoring_changemodifier extends  MethodInfoAst$$typesys  {

    // Sucht den Node AST_Modifiers und bergibt ihn.
    // Wenn dieser nicht existiert, wird er erstellt und in den AST eingebunden.
    public AST_Modifiers getAstModifiers() {

        AST_Modifiers tmpModifiers = getModifiersASTNode();
        if (tmpModifiers != null) {
            return tmpModifiers;
        }

        // Wenn der Knoten nicht vorhanden ist erstelle ihn und binde ihn ein
        AstOptNode tmpNode = (AstOptNode) astClassBodyDcl.arg[0];
        tmpNode.arg[0] = new AST_Modifiers();
        tmpNode.arg[0].up = tmpNode;
        tmpNode.arg[0].arg = new AstNode[] { null };
        return (AST_Modifiers) tmpNode.arg[0];
    }

    //ndert nur die Sichtbarkeits-Modifier (und entfernt ggf. den abstract
    //Modifier, wenn auf private gendert wird)
    public void changeVisibility(ModifierInfo[] someModifier)
    throws TransformationException{

	changeVisibilityLocal(someModifier);

	boolean isPackagePrivate = true;
	boolean deleteAbstract = false;
	AST_Modifiers astModifiersNode = getAstModifiers();

	AstCursor cursor = new AstCursor();
	if(astModifiersNode.arg!=null && astModifiersNode.arg[0]!=null){
	    for (cursor.First(astModifiersNode); cursor.More(); cursor.PlusPlus()) {
		//Wenn schon ein Sichtbarkeits-Modifier vorhanden war
		if(ChangeModifier.isAccessModifier(cursor.node)){
		    isPackagePrivate=false;
		    String whitespace = ((AstToken)cursor.node.tok[0]).white_space;

		    //wenn auf packageprivate gewechselt werden soll
		    if(someModifier[0]==null){
			//nchste AstToken suchen um whitespace zu setzen
			AstCursor c = new AstCursor();
			AstNode start = cursor.node.up.right!= null ? cursor.node.up.right : astModifiersNode.up.right;
			for (c.First(start); c.More(); c.PlusPlus()) {
			    if(c.node.tok != null && c.node.tok[0]!=null){
				((AstToken)c.node.tok[0]).white_space = whitespace;
				break;
			    }
			}
			//Delete the Modifier
			ModifierFactory.deleteNode(cursor.node.up);
			return;
		    }
		    else
			if(someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PUBLIC))){
			    cursor.node.up.arg[0] = ModifierFactory.getModifier((Modified)((MethodInfoAst) ((MethodInfoAst) this)),"public",whitespace);
			    cursor.node.up.arg[0].up = cursor.node.up;
			}
			else
			    if(someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PROTECTED))){
				cursor.node.up.arg[0] = ModifierFactory.getModifier((Modified)((MethodInfoAst) ((MethodInfoAst) this)),"protected",whitespace);
				cursor.node.up.arg[0].up = cursor.node.up;
			    }
			    else
				if(someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PRIVATE))){
				    cursor.node.up.arg[0] = ModifierFactory.getModifier((Modified)((MethodInfoAst) ((MethodInfoAst) this)),"private",whitespace);
				    cursor.node.up.arg[0].up = cursor.node.up;
				}
		}
		//Wenn die Methode abstract war und auf private gendert werden soll,
		//darf die Methode nicht abstract bleiben
		else if ( cursor.node instanceof ModAbstract && someModifier[0]!=null &&
			someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PRIVATE))){
		    deleteAbstract = true;
		}
	    }
	}

	if(deleteAbstract){
	    cursor = new AstCursor();
	    for (cursor.First(astModifiersNode); cursor.More(); cursor.PlusPlus()) {
		if ( cursor.node instanceof ModAbstract){
		    ModifierFactory.deleteNode(cursor.node.up);
		    break;
		}
	    }
	}

	//Wenn die Methode packageprivate war muss ein neuer AST_ModifiersElem-Knoten
	//erstellt werden
	if(isPackagePrivate){

	    //aber nur wenn nicht auf packageprivate gewechselt erden soll
	    if(someModifier[0] == null)
		return;

	    String whitespace = "";
	    //find next AstToken for whitespace
	    AstCursor c = new AstCursor();
	    for (c.First(astModifiersNode.up.up); c.More(); c.PlusPlus()) {
		if(c.node.tok != null && c.node.tok[0]!=null){
		    whitespace = ((AstToken)c.node.tok[0]).white_space;
		    ((AstToken)c.node.tok[0]).white_space = " ";
		    break;
		}
	    }

	    AST_ModifiersElem astModifiersElemNode = new AST_ModifiersElem();

	    if(someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PUBLIC)))
		astModifiersElemNode.arg = new AstNode[] {ModifierFactory.getModifier((Modified)((MethodInfoAst) ((MethodInfoAst) this)),"public",whitespace)};
	    else if(someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PROTECTED)))
		astModifiersElemNode.arg = new AstNode[] {ModifierFactory.getModifier((Modified)((MethodInfoAst) ((MethodInfoAst) this)),"protected",whitespace)};
	    else if(someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PRIVATE)))
		astModifiersElemNode.arg = new AstNode[] {ModifierFactory.getModifier((Modified)((MethodInfoAst) ((MethodInfoAst) this)),"private",whitespace)};

	    astModifiersElemNode.arg[0].up = astModifiersElemNode;
	    ModifierFactory.insertAsFirst(astModifiersNode,astModifiersElemNode);
	}
    }

    //ndert nur die Sichtbarkeits-Modifier des modifier-Felds des MethodInfoAst
    //(und entfernt ggf. den abstract-Modifier, wenn auf private gendert wird)
    private void changeVisibilityLocal(ModifierInfo[] someModifier)
    throws TransformationException {

	//Eine Methode kann nur einen Sichtbarkeits-Modifier besitzen
	if(someModifier.length!=1)
	    throw new TransformationException("only one access modifier is allowed.");

	//nur Sichtbarkeits-Modifier sind in dieser Methode zugelassen
	if(!ChangeModifier.isAccessModifier(someModifier[0]))
	    throw new TransformationException(someModifier[0] +
	    " is no access modifier.");

	//wenn es vorher keine Modifier gab
	if(modifier == null){
	    if(someModifier[0]!=null)
		modifier = someModifier;
	    return;
	}

	int removedModifier = 0;
	boolean isPackagePrivate = true;    		

	for(int j=0; j<modifier.length; j++)
	    if(modifier[j] != null)
		//Wenn schon ein Sichtbarkeits-Modifier vorhanden war
		if(ChangeModifier.isAccessModifier(modifier[j])){
		    modifier[j] = someModifier[0];
		    isPackagePrivate=false;
		    //Wenn die Methode abstract war und auf private gendert werden soll,
		    //darf die Methode nicht abstract bleiben
		} else if(someModifier[0]!=null && 
			someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PRIVATE)) &&
			modifier[j].equals(ModifierInfo.valueOf(ModifierInfo.ABSTRACT))){
		    modifier[j] = null;
		    removedModifier++;
		}

	if(removedModifier==0 && isPackagePrivate && someModifier[0]==null)
	    return;

	if(isPackagePrivate || removedModifier!=0){

	    int length;
	    if(isPackagePrivate)
		length = modifier.length+1-removedModifier;
	    else 
		length = modifier.length-removedModifier;

	    ModifierInfo[] tmpModifier = new ModifierInfo[length];

	    int i=0;
	    for(int j=0; j<modifier.length; j++){
		if(modifier[j]!=null)
		    tmpModifier[i++] = modifier[j];
	    }
	    if(isPackagePrivate)
		tmpModifier[i] = someModifier[0];

	    modifier = tmpModifier;
	}
    }

	public void changeModifier(ModifierInfo[] someModifier)
        throws TransformationException {}
      // inherited constructors


    
    /**
     * Konstruktor.
     * 
     * @param aDecl
     *            MethodDcl AST Knoten.
     */
    public MethodInfoAst$$refactoring_changemodifier ( ClassBodyDeclaration aDecl ) { super(aDecl); }
}




/**
 * @layer<refactoring_extrmethod>
 * @author Stephan Klapproth
 * 
 */
 class MethodInfoAst extends  MethodInfoAst$$refactoring_changemodifier  {

    public void remove() {
        astClassBodyDcl.Delete();
        ((ClassInfoAst) getContext()).removeMethod(signature());
    }

    ClsBody getClassBody(ClassInfoAst aDestClass) {
        TypeDeclaration tmpDecl = ClassInfoAst.getFirstClass((program) aDestClass
                .getAstNode());
        AstCursor c = new AstCursor();
        for (c.First(tmpDecl); c.More(); c.PlusPlus()) {
            if (c.node instanceof ClsBody) {
                return (ClsBody) c.node;
            }
        }
        return null;
    }

    public void move(ClassInfoAst aDestClass){

		////update the AST and the ClassInfo of the Source-Class

		//get the number of empty lines before the method (which will also be extracted)
        AstCursor c = new AstCursor();
        int begin = ((ClassInfoAst)((MethodInfoAst) this).getContext()).startLineNum();
        for (c.First(getClassBody((ClassInfoAst)((MethodInfoAst) this).getContext()).arg[0]); c.More(); c.PlusPlus()) {
            if (c.node == astClassBodyDcl)
        		break;

            if (c.node.tok!=null && c.node.tok.length!=0){
        		for (int i=0; i<c.node.tok.length; i++){
        	    	if (c.node.tok[i] instanceof AstToken){
        				AstToken t = (AstToken)c.node.tok[i];
        					if (t.lineNum()>begin)
        		   				begin = t.lineNum();
        	    	}
        		}
            }
        }
		int numOfEmptyLines = startLineNum()-begin;

		//temporarily save lineinformation
		begin = endLineNum();
		int methodBeginOld = startLineNum();
		int delta = endLineNum() - (startLineNum()-numOfEmptyLines);

		//delete old references to this method
        ModifierFactory.deleteNode((AST_FieldDeclElem)((MethodInfoAst) this).astClassBodyDcl.up);
        ((ClassInfoAst) getContext()).removeMethod(signature());

        //update all linenumbers representing code, located in the classfile
		//after this method
    	for (c.First(((ClassInfoAst)getContext()).getAstNode()); c.More(); c.PlusPlus()) {
    	    if (c.node.tok!=null && c.node.tok.length!=0){
    			for (int i=0; i<c.node.tok.length; i++){
    		    	if (c.node.tok[i] instanceof AstToken){
    					AstToken t = (AstToken)c.node.tok[i];
    					if (t.lineNum()>= begin)
    			    		t.setParms(t.white_space, t.name, t.lineNum()-delta);
    		    	}
    			}
    	    }
    	}

    	//set scope of this method
    	((MethodInfoAst) this).setContext(aDestClass);


        ////update the AST and the ClassInfo of the Destination-Class

    	//get the ClsBody-Node and from there the AST_FieldDecl-Node 
		//of the Destination-Class (and if it not exists, create it)
        ClsBody tmpClsBody = getClassBody(aDestClass);
        AST_FieldDecl tmpFieldDecl = tmpClsBody.getAST_FieldDecl();
        if (tmpFieldDecl == null) {
            tmpClsBody.arg[0].arg[0] = new AST_FieldDecl();
            tmpClsBody.arg[0].arg[0].up = tmpClsBody.arg[0];
            tmpFieldDecl = tmpClsBody.getAST_FieldDecl();
        }

        //find the linenumber of the destination class where the method will
        //be inserted
        begin = aDestClass.startLineNum();
        for (c.First(getClassBody(aDestClass).arg[0]); c.More(); c.PlusPlus()) {
            if (c.node.tok!=null && c.node.tok.length!=0){
        		for (int i=0; i<c.node.tok.length; i++){
        	    	if (c.node.tok[i] instanceof AstToken){
        				AstToken t = (AstToken)c.node.tok[i];
        				if (t.lineNum()>begin)
        		    		begin = t.lineNum();
        	    	}
        		}
            }
        }

        //update the linenumbers of this method
        int delta2 = begin-methodBeginOld+numOfEmptyLines;
        for (c.First(((MethodInfoAst) this).astClassBodyDcl); c.More(); c.PlusPlus()) {
            if (c.node.tok!=null && c.node.tok.length!=0){
        		for (int i=0; i<c.node.tok.length; i++){
        	    	if (c.node.tok[i] instanceof AstToken){
        				AstToken t = (AstToken)c.node.tok[i];
        				t.setParms(t.white_space, t.name, t.lineNum()+delta2);
        	    	}
        		}
            }
        }

        //use the location to update all linenumbers representing code,
        //located in the destination-file after the inserted method (normally just
        //the end of the class)
    	for (c.First(aDestClass.getAstNode()); c.More(); c.PlusPlus()) {
    	    if (c.node.tok!=null && c.node.tok.length!=0){
    			for (int i=0; i<c.node.tok.length; i++){
    		    	if (c.node.tok[i] instanceof AstToken){
    					AstToken t = (AstToken)c.node.tok[i];
    					if (t.lineNum()>begin)
    			    		t.setParms(t.white_space, t.name, t.lineNum()+delta);
    		    	}
    			}
    	    }
    	}

    	//add the MethodDcl-Node of the extracted Method to the AST_FieldDecl-Node
        //of the Destination-Class and update the ClassInfo of the Destination-Class
    	tmpFieldDecl.add((AST_FieldDeclElem) astClassBodyDcl.up);
    	aDestClass.addDeclaration(((MethodInfoAst) ((MethodInfoAst) this)));
    }
      // inherited constructors


    
    /**
     * Konstruktor.
     * 
     * @param aDecl
     *            MethodDcl AST Knoten.
     */
    public MethodInfoAst ( ClassBodyDeclaration aDecl ) { super(aDecl); }

}
