package refactor;



/**
 * Klassen, die <code>Scope</code> implementieren, beschreiben ein
 * Javaelement, welches Deklarationen aufnehmen kann.
 * 
 * @layer typesys
 * @author Stephan Klapproth
 * 
 */
public interface Scope extends Named {

    /**
     * Fgt dem <code>Scope</code> eine neue Deklaration hinzu.
     * 
     * @param aDeclaration
     */
    public void addDeclaration(Declaration aDeclaration);

    /**
     * Gibt den <code>Scope</code> zurck, von dem der aktuelle
     * <code>Scope</code> umgeben ist.
     * 
     * @return <code>Scope</code>, von dem dieser <code>Scope</code>
     *         umgeben ist.
     */
    public Scope getContext();

    /**
     * Setzt den <code>Scope</code>, der diesen <code>Scope</code> umgibt.
     * 
     * @param aScope
     *            <code>Scope</code>, in dem sich diese <code>Scope</code>
     *            Instanz befindet.
     */
    public void setContext(Scope aScope);

    /**
     * Sucht nach einem Feld mit dem Namen <code>aName<code/>.
     * 
     * <p> Sollte kein Feld mit dem Namen existieren, 
     * wird <code>null</code> zurckgegeben.
     * 
     * @param aName 
     *            der Name des gesuchten Felds.
     * @return FieldInfo 
     *            Instanz, die das Feld mit dem Namen <code>aName</code> 
     *            reprsentiert.
     */
    public FieldInfo findField(String aName);

}
